@@ -3,6 +3,7 @@ Implements the YAMLValueFormats enumeration.
 
 Copyright 2019, 2020 William W. Kimball, Jr. MBA MSIS
 """
+import datetime
 from enum import Enum, auto
 from typing import Any, List
 
@@ -16,6 +17,20 @@ from ruamel.yaml.scalarstring import (
 from ruamel.yaml.scalarbool import ScalarBoolean
 from ruamel.yaml.scalarfloat import ScalarFloat
 from ruamel.yaml.scalarint import ScalarInt
+# pylint: disable=wrong-import-position,ungrouped-imports
+from ruamel.yaml import version_info as ryversion
+if ryversion < (0, 17, 22):                   # pragma: no cover
+    from yamlpath.patches.timestamp import (
+        AnchoredTimeStamp,
+        AnchoredDate,
+    )  # type: ignore
+else:                                         # pragma: no cover
+    # Temporarily fool MYPY into resolving the future-case imports
+    from ruamel.yaml.timestamp import TimeStamp as AnchoredTimeStamp
+    AnchoredDate = AnchoredTimeStamp
+    #from ruamel.yaml.timestamp import AnchoredTimeStamp
+    # From whence shall come AnchoredDate?
+# pylint: enable=wrong-import-position,ungrouped-imports
 
 
 class YAMLValueFormats(Enum):
@@ -32,6 +47,9 @@ class YAMLValueFormats(Enum):
     `BOOLEAN`
         The value is written as a bare True or False.
 
+    `DATE`
+        The value is written as a bare ISO8601 date without a time component.
+
     `DEFAULT`
         The value is written in whatever format is deemed most appropriate.
 
@@ -56,10 +74,15 @@ class YAMLValueFormats(Enum):
 
     `SQUOTE`
         The value is demarcated via apostrophes (').
+
+    `TIMESTAMP`
+        The value is a timestamp per the supported syntax of ISO8601 by
+        http://yaml.org/type/timestamp.html.
     """
 
     BARE = auto()
     BOOLEAN = auto()
+    DATE = auto()
     DEFAULT = auto()
     DQUOTE = auto()
     FLOAT = auto()
@@ -67,6 +90,7 @@ class YAMLValueFormats(Enum):
     INT = auto()
     LITERAL = auto()
     SQUOTE = auto()
+    TIMESTAMP = auto()
 
     @staticmethod
     def get_names() -> List[str]:
@@ -137,5 +161,9 @@ class YAMLValueFormats(Enum):
             best_type = YAMLValueFormats.FLOAT
         elif node_type is ScalarInt:
             best_type = YAMLValueFormats.INT
+        elif node_type is AnchoredDate or node_type is datetime.date:
+            best_type = YAMLValueFormats.DATE
+        elif node_type is AnchoredTimeStamp or node_type is datetime.datetime:
+            best_type = YAMLValueFormats.TIMESTAMP
 
         return best_type
