@@ -7,7 +7,7 @@ import re
 from subprocess import run, PIPE, CalledProcessError
 from os import access, sep, X_OK
 from shutil import which
-from typing import Any, Generator, List, Optional
+from typing import Any, Generator, List, Optional, Union
 
 from ruamel.yaml.comments import CommentedSeq, CommentedMap
 
@@ -16,7 +16,7 @@ from yamlpath.common import Anchors
 from yamlpath.eyaml.enums import EYAMLOutputFormats
 from yamlpath.enums import YAMLValueFormats, PathSeperators
 from yamlpath.eyaml.exceptions import EYAMLCommandException
-from yamlpath.wrappers import ConsolePrinter
+from yamlpath.wrappers import ConsolePrinter, NodeCoords
 from yamlpath import Processor
 
 
@@ -112,7 +112,9 @@ class EYAMLProcessor(Processor):
         for path in self._find_eyaml_paths(self.data, YAMLPath()):
             yield path
 
-    def decrypt_eyaml(self, value: str) -> str:
+    def decrypt_eyaml(
+        self, value: Union[str, list, NodeCoords]
+    ) -> Union[str, list]:
         """
         Decrypt an EYAML value.
 
@@ -125,6 +127,16 @@ class EYAMLProcessor(Processor):
         Raises:
         - `EYAMLCommandException` when the eyaml binary cannot be utilized
         """
+        if isinstance(value, NodeCoords):
+            value = self.decrypt_eyaml(value.node)
+
+        # Recursively decrypt lists
+        if isinstance(value, list):
+            decvals: list = []
+            for encval in value:
+                decvals.append(self.decrypt_eyaml(encval))
+            return decvals
+
         if not self.is_eyaml_value(value):
             return value
 
@@ -247,7 +259,8 @@ class EYAMLProcessor(Processor):
             )
 
         if output is EYAMLOutputFormats.BLOCK:
-            retval = re.sub(r" +", "", retval) + "\n"
+            fixval: str = re.sub(r" ", "", retval.strip())
+            retval = re.sub(r"\r\n", " ", fixval) + "\n"
 
         self.logger.debug(
             f"Encrypted result:\n{retval}",
@@ -294,7 +307,7 @@ class EYAMLProcessor(Processor):
     def get_eyaml_values(
         self, yaml_path: YAMLPath, mustexist: bool = False,
         default_value: str = ""
-    ) -> Generator[str, None, None]:
+    ) -> Generator[Union[str, list], None, None]:
         """
         Retrieve and decrypt all EYAML nodes identified via a YAML Path.
 
@@ -316,7 +329,7 @@ class EYAMLProcessor(Processor):
         self.logger.verbose(f"Decrypting value(s) at {yaml_path}.")
         for node in self.get_nodes(yaml_path, mustexist=mustexist,
                                    default_value=default_value):
-            plain_text: str = self.decrypt_eyaml(node.node)
+            plain_text: Union[str, list] = self.decrypt_eyaml(node.node)
             yield plain_text
 
     def _can_run_eyaml(self) -> bool:
